/*******************************************************************************
 * Copyright (c) 2004-2010 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/

package hu.bme.mit.ftsrg.szilvas;

import hu.bme.mit.ftsrg.szilvas.model.GyumolcsosTal;
import hu.bme.mit.ftsrg.szilvas.model.KonyhaPult;
import hu.bme.mit.ftsrg.szilvas.model.Kukac;
import hu.bme.mit.ftsrg.szilvas.model.SzemetesKuka;
import hu.bme.mit.ftsrg.szilvas.model.Szilva;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;

/**
 * @author Bergmann Gábor
 *
 */
public class SzilvaTest {
	private static void szilvak(StatefulKnowledgeSession ksession) {
		Szilva sz = new Szilva(3, false);
		sz.berak(new Kukac());
		insert(ksession, sz);
		
		Szilva sz2 = new Szilva(8, false);
		sz2.berak(new Kukac());
		insert(ksession, sz2);
		
		insert(ksession, new Szilva(4, true));
		insert(ksession, new Szilva(5, false));
		insert(ksession, new Szilva(6, false));
		insert(ksession, new Szilva(7, true));
		insert(ksession, new Szilva(1, false));
		insert(ksession, new Szilva(2, false));
		insert(ksession, new Szilva(3, false));
	}
	private static void konyha(StatefulKnowledgeSession ksession) {
		GyumolcsosTal gyumolcsosTal = new GyumolcsosTal();
		FactHandle talHandle = ksession.insert(gyumolcsosTal);
		SzemetesKuka szemetesKuka = new SzemetesKuka();
		FactHandle kukaHandle = ksession.insert(szemetesKuka);
		KonyhaPult konyhaPult = new KonyhaPult();
		FactHandle pultHandle = ksession.insert(konyhaPult);
		
		szilvak(ksession);
		
		ksession.fireAllRules();
		System.out.println("---");
		System.out.println("Kuka: " + szemetesKuka);
		System.out.println("Tál: " + gyumolcsosTal);
		System.out.println("Pult: " + konyhaPult);
	}
	
	public static final void main(String[] args) {
		try {
			// load up the knowledge base
			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newFileLogger(ksession, "logs/szilva");
			// go !
			
			konyha(ksession);
			logger.close();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	
	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("gombocgyuras.drl"), ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error: errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}
	private static void insert(StatefulKnowledgeSession ksession, Szilva sz) {
		ksession.insert(sz);
		for (Object o : sz.getTartalom())
			ksession.insert(o);	
	}
}
